import React from 'react';
import PropTypes from 'prop-types';

import amapConfig from './config/amap';
import gisModules from './config/module';

import AMapTileLayer from './extensions/layers/AMapTileLayer';
import GoogleTileLayer from './extensions/layers/GoogleTileLayer';

import * as utils from './utils/utils';

export default class MapManager extends React.Component {
  static defaultProps = {
    id: '#jsmap',
    width: '100%',
    height: '420px',
    mapsettings: {},
    modules: gisModules,
  };

  constructor(props) {
    super(props);
    this.mapRef = React.createRef();
    this.initializeVariables();
  }

  initializeVariables() {
    this.view = null;
    this.map = null;
    this.basemap = null;
    this.tipManager = null;
    this.AreaLayerSum = null;
    this.mapsettings = this.props.mapsettings;
    window._config = {
      site: '',
    };
  }

  getStyle() {
    return {
      width: this.props.width,
      height: this.props.height,
    };
  }

  componentDidMount() {
    utils.loadModules(this.props.modules).then(
      function (moduleArr) {
        this.getModules(moduleArr);
        this.createMapLayers();
        this.renderView();
      }.bind(this),
    );
  }

  getModules(moduleArr) {
    this.loadedModules = {
      Map: moduleArr[0],
      BaseMap: moduleArr[1],
      MapView: moduleArr[2],
      SceneView: moduleArr[3],
      TileLayer: moduleArr[4],
      MapImageLayer: moduleArr[5],
      ElevationLayer: moduleArr[6],
      Ground: moduleArr[7],
      SceneLayer: moduleArr[8],
      FeatureLayer: moduleArr[9],
      GroupLayer: moduleArr[10],
      VectorTileLayer: moduleArr[11],
      OpenStreetMapLayer: moduleArr[12],
      Extent: moduleArr[13],
      Collection: moduleArr[14],
      Camera: moduleArr[15],
      Viewpoint: moduleArr[16],
      jsonUtils: moduleArr[17],
      watchUtils: moduleArr[18],
      Graphic: moduleArr[19],
      PictureMarkerSymbol: moduleArr[20],
      Popup: moduleArr[21],
      GraphicsLayer: moduleArr[22],
      matrix: moduleArr[23],
      Color: moduleArr[24],
      SimpleFillSymbol: moduleArr[25],
      SimpleLineSymbol: moduleArr[26],
      TextSymbol: moduleArr[27],
      Point: moduleArr[28],
      lang: moduleArr[29],
      promiseUtils: moduleArr[30],
    };
  }

  createMapLayers() {
    this.loadedModules.AMapTileLayer = AMapTileLayer(
      this.loadedModules.TileLayer,
      this.loadedModules.lang,
      this.loadedModules.promiseUtils,
    );
    this.loadedModules.GoogleTileLayer = GoogleTileLayer(
      this.loadedModules.TileLayer,
      this.loadedModules.lang,
      this.loadedModules.promiseUtils,
    );
  }

  renderView() {
    this.addLayerProxy();
    this.addMap();
    this.addBasemaps();
  }

  addLayerProxy() {}

  addMap() {
    const mapsettings = this.mapsettings;
    this.map = new this.loadedModules.Map();
    if (this.mapsettings.ground) {
      if ('string' == typeof this.mapsettings.ground) {
        this.map.ground = this.mapsettings.ground;
      } else {
        var layers = mapsettings.ground.layers,
          layer,
          groundlayers = new this.loadedModules.Collection();
        layers.forEach(
          function (layerconfig) {
            this.createLayer(layerconfig, groundlayers, false, '');
          }.bind(this),
        );
        this.map.ground = new Ground({
          layers: groundlayers,
        });
      }
    }
    if ('2d' == mapsettings.type.toLowerCase()) {
      this.view = new this.loadedModules.MapView({
        map: this.map,
        container: this.mapRef.current,
        manager: this,
        resizeAlign: mapsettings.resizeAlign || 'top-left',
      });
    } else if ('3d' == mapsettings.type.toLowerCase()) {
      this.view = new this.loadedModules.SceneView({
        map: this.map,
        manager: this,
        qualityProfile: mapsettings.qualityProfile || 'high',
        container: this.mapRef.current,
      });
      if (mapsettings.viewingMode) {
        this.view.viewingMode = mapsettings.viewingMode;
      }
      if (mapsettings.environment) {
        this.view.environment = mapsettings.environment;
      }
      if (mapsettings.clippingArea) {
        this.view.clippingArea = new Extent(mapsettings.clippingArea);
      }
      if (mapsettings.extent) {
        this.view.extent = new Extent(mapsettings.extent);
      }
    }
    //设置View初始范围
    this.viewpoint = mapsettings.viewpoint;
    this.setInitialViewpoint = this.setInitialViewpoint.bind(this);
    this.view.then(this.setInitialViewpoint);
  }

  setInitialViewpoint() {}

  addBasemaps() {
    let basemapconfigs = this.mapsettings.basemaps;
    this.basemaps = [];
    if (basemapconfigs) {
      basemapconfigs.forEach(
        function (basemapconfig) {
          this.basemaps.push(this.createBasemap(basemapconfig));
        }.bind(this),
      );
    } else {
      this.basemaps.push(this.createDefaultBasemap());
    }

    if (this.basemaps.length) {
      this.map.basemap = this.basemaps[0];
    }
  }

  createBasemap(basemap) {
    var bm;
    if (basemap) {
      if ('string' == typeof basemap) {
        bm = basemap;
      } else {
        // var layer,
        var bls = new this.loadedModules.Collection(),
          id = basemap.id,
          title = basemap.title,
          thumbnailUrl = basemap.thumbnailUrl,
          baseLayers = basemap.baseLayers;
        baseLayers.forEach(
          function (layerconfig) {
            this.createLayer(layerconfig, bls, false, 'baseMap');
          }.bind(this),
        );
        bm = new this.loadedModules.BaseMap({
          id: id,
          title: title,
          thumbnailUrl: thumbnailUrl,
          baseLayers: bls,
        });
      }
    }
    return bm;
  }

  createDefaultBasemap() {}

  createLayer(config, collection, returnLayer, layerType) {
    var layer;
    var layers = [];
    if (config.url) {
      //config.url = urlUtils.getUrl(config.url);
    }
    switch (config.layerType.toLowerCase()) {
      case 'tilelayer':
        layer = new this.loadedModules.TileLayer(config);
        break;
      case 'pipenetlayer':
      case 'mapimagelayer':
        layer = new this.loadedModules.MapImageLayer(
          utils._.extend({}, config, {
            token: _config.token,
            client: 'webgis',
            visible: false,
          }),
        );
        layer.then(
          function () {
            if (layer.hasAnnotation) {
              //  layer.annotationLayer = this.addAnnotationLayer(layer.title, layer.url, config.visible);
            }
          }.bind(this),
        );
        break;
      case 'elevationlayer':
        layer = new this.loadedModules.ElevationLayer(config);
        break;
      case 'scenelayer':
        layer = new this.loadedModules.SceneLayer(config);
        break;
      case 'featurelayer':
        layer = new this.loadedModules.FeatureLayer(config);
        break;
      case 'grouplayer':
        var layers = config.layers.map(function (c) {
          return this.createLayer(c, [], true, '');
        }, this);
        layer = new this.loadedModules.GroupLayer(
          utils._.extend({}, config, {
            layers: layers,
          }),
        );
        break;
      case 'google-v':
      case 'google-i':
      case 'google-i-a':
      case 'google-ia':
        layer = new this.loadedModules.GoogleTileLayer(
          utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'google-user':
        var proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new this.loadedModules.UserTileLayer(
          utils._.extend(config, {
            url: proxyUrl,
          }),
        );
        layer.then((res) => console.log(res));
        break;
      case 'pipenet-tile':
        var proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new this.loadedModules.PipenetTileLayer(
          utils._.extend(config, {
            url: proxyUrl,
          }),
        );
        break;
      case 'baidu-v':
      case 'baidu-i':
      case 'baidu-ia':
        layer = new this.loadedModules.BaiduTileLayer(
          utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'amap-v':
      case 'amap-i':
      case 'amap-ia':
      case 'amap-v2':
      case 'amap-i2':
      case 'amap-ia2':
        layer = new this.loadedModules.AMapTileLayer(
          utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v':
      case 'tianditu-va':
      case 'tianditu-i':
      case 'tianditu-ia':
      case 'tianditu-t':
      case 'tianditu-ta':
      case 'tianditu-ll-v':
      case 'tianditu-ll-va':
      case 'tianditu-ll-i':
      case 'tianditu-ll-ia':
      case 'tianditu-ll-t':
      case 'tianditu-ll-ta':
        layer = new this.loadedModules.TiandituTileLayer(
          utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v-va':
        //深拷贝config
        var configString = JSON.stringify(config);
        //天地图矢量
        var configv = utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-v',
        });
        this.createLayer(configv, collection, false, layerType);

        //天地图注记
        var configva = utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-va',
        });
        this.createLayer(configva, collection, false, layerType);
        break;
      case 'tianditu-i-ia':
        //深拷贝config
        var configString = JSON.stringify(config);
        //天地图矢量
        var configi = utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-i',
        });
        this.createLayer(configi, collection, false, layerType);

        //天地图注记
        var configia = utils._.extend(JSON.parse(configString), {
          layerType: 'tianditu-ia',
        });
        this.createLayer(configia, collection, false, layerType);

        break;
      case 'osm':
        layer = new this.loadedModules.OpenStreetMapLayer(
          utils._.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'wmts-v':
        layer = new this.loadedModules.WMTSTileLayer(utils._.extend(config, {}));
        break;
      case 'arcgis-streetpurplishblue':
      case 'arcgis-streetwarm':
      case 'arcgis-community':
      case 'arcgis-streetgray':
        layer = new this.loadedModules.TileLayer(
          utils._.extend(config, {
            url:
              'http://map.geoq.cn/ArcGIS/rest/services/ChinaOnline' +
              config.layerType.replace('arcgis-', '') +
              '/MapServer',
          }),
        );
        break;
      case 'arcgis-tilelayer':
        if (_config.ip.indexOf('222.92') > -1)
          config.url = config.url.replace(/\d+\.\d+\.\d+\.\d+/, window.location.hostname);
        layer = new this.loadedModules.TileLayer({
          url: config.url,
          title: config.title,
          //    id: _.uniqueId('arcgis-tilelayer_')
        });
        break;
      default:
        if (config.layerType.toLowerCase().indexOf('tianditu-') > -1) {
          layer = new this.loadedModules.TiandituTileLayer(
            utils._.extend(config, {
              url: 'http://fack.com',
            }),
          );
          break;
        }
        break;
    }
    //returnLayer 是否需要返回layer
    if (returnLayer) return layer;

    //layer未给值
    if (_.isUndefined(layer)) return this._layerCreateErrorLog(config);

    //layerType
    switch (layerType) {
      case 'baseMap':
        layer.visible = true;
        break;
      case 'pipeLayer':
        utils._.extend(layer, {
          preload: true,
        });
        break;
      default:
        break;
    }
    collection.add(layer);
  }

  render() {
    return <div ref={this.mapRef} id={this.props.id} style={this.getStyle()}></div>;
  }
}

MapManager.propTypes = {
  id: PropTypes.string,
  width: PropTypes.string,
  height: PropTypes.string,
  mapsettings: PropTypes.object,
  modules: PropTypes.array,
};
